@@ -1,5 +1,7 @@
 """Quickly setup documentation source to work with Sphinx."""
 
+from __future__ import annotations
+
 import argparse
 import locale
 import os
@@ -7,14 +9,14 @@ import sys
 import time
 from collections import OrderedDict
 from os import path
-from typing import TYPE_CHECKING, Any, Callable, Dict, List, Union
+from typing import TYPE_CHECKING, Any, Callable
 
 # try to import readline, unix specific enhancement
 try:
     import readline
     if TYPE_CHECKING and sys.platform == "win32":  # always false, for type checking
         raise ImportError
-
+    READLINE_AVAILABLE = True
     if readline.__doc__ and 'libedit' in readline.__doc__:
         readline.parse_and_bind("bind ^I rl_complete")
         USE_LIBEDIT = True
@@ -22,7 +24,7 @@ try:
         readline.parse_and_bind("tab: complete")
         USE_LIBEDIT = False
 except ImportError:
-    readline = None
+    READLINE_AVAILABLE = False
     USE_LIBEDIT = False
 
 from docutils.utils import column_width
@@ -130,10 +132,12 @@ def ok(x: str) -> str:
     return x
 
 
-def do_prompt(text: str, default: str = None, validator: Callable[[str], Any] = nonempty) -> Union[str, bool]:  # NOQA
+def do_prompt(
+    text: str, default: str | None = None, validator: Callable[[str], Any] = nonempty
+) -> str | bool:
     while True:
         if default is not None:
-            prompt = PROMPT_PREFIX + '%s [%s]: ' % (text, default)
+            prompt = PROMPT_PREFIX + f'{text} [{default}]: '
         else:
             prompt = PROMPT_PREFIX + text + ': '
         if USE_LIBEDIT:
@@ -141,7 +145,7 @@ def do_prompt(text: str, default: str = None, validator: Callable[[str], Any] =
             # sequence (see #5335).  To avoid the problem, all prompts are not colored
             # on libedit.
             pass
-        elif readline:
+        elif READLINE_AVAILABLE:
             # pass input_mode=True if readline available
             prompt = colorize(COLOR_QUESTION, prompt, input_mode=True)
         else:
@@ -159,8 +163,8 @@ def do_prompt(text: str, default: str = None, validator: Callable[[str], Any] =
 
 
 class QuickstartRenderer(SphinxRenderer):
-    def __init__(self, templatedir: str) -> None:
-        self.templatedir = templatedir or ''
+    def __init__(self, templatedir: str = '') -> None:
+        self.templatedir = templatedir
         super().__init__()
 
     def _has_custom_template(self, template_name: str) -> bool:
@@ -170,12 +174,9 @@ class QuickstartRenderer(SphinxRenderer):
               It will be removed in the future without deprecation period.
         """
         template = path.join(self.templatedir, path.basename(template_name))
-        if self.templatedir and path.exists(template):
-            return True
-        else:
-            return False
+        return bool(self.templatedir) and path.exists(template)
 
-    def render(self, template_name: str, context: Dict) -> str:
+    def render(self, template_name: str, context: dict[str, Any]) -> str:
         if self._has_custom_template(template_name):
             custom_template = path.join(self.templatedir, path.basename(template_name))
             return self.render_from_file(custom_template, context)
@@ -183,7 +184,7 @@ class QuickstartRenderer(SphinxRenderer):
             return super().render(template_name, context)
 
 
-def ask_user(d: Dict[str, Any]) -> None:
+def ask_user(d: dict[str, Any]) -> None:
     """Ask the user for quickstart values missing from *d*.
 
     Values are:
@@ -226,7 +227,7 @@ def ask_user(d: Dict[str, Any]) -> None:
         d['path'] = do_prompt(__('Please enter a new root path (or just Enter to exit)'),
                               '', is_path_or_empty)
         if not d['path']:
-            sys.exit(1)
+            raise SystemExit(1)
 
     if 'sep' not in d:
         print()
@@ -237,9 +238,9 @@ def ask_user(d: Dict[str, Any]) -> None:
 
     if 'dot' not in d:
         print()
-        print(__('Inside the root directory, two more directories will be created; "_templates"\n'      # NOQA
-                 'for custom HTML templates and "_static" for custom stylesheets and other static\n'    # NOQA
-                 'files. You can enter another prefix (such as ".") to replace the underscore.'))       # NOQA
+        print(__('Inside the root directory, two more directories will be created; "_templates"\n'      # noqa: E501
+                 'for custom HTML templates and "_static" for custom stylesheets and other static\n'    # noqa: E501
+                 'files. You can enter another prefix (such as ".") to replace the underscore.'))       # noqa: E501
         d['dot'] = do_prompt(__('Name prefix for templates and static dir'), '_', ok)
 
     if 'project' not in d:
@@ -262,12 +263,14 @@ def ask_user(d: Dict[str, Any]) -> None:
 
     if 'language' not in d:
         print()
-        print(__('If the documents are to be written in a language other than English,\n'
-                 'you can select a language here by its language code. Sphinx will then\n'
-                 'translate text that it generates into that language.\n'
-                 '\n'
-                 'For a list of supported codes, see\n'
-                 'https://www.sphinx-doc.org/en/master/usage/configuration.html#confval-language.'))  # NOQA
+        print(__(
+            'If the documents are to be written in a language other than English,\n'
+            'you can select a language here by its language code. Sphinx will then\n'
+            'translate text that it generates into that language.\n'
+            '\n'
+            'For a list of supported codes, see\n'
+            'https://www.sphinx-doc.org/en/master/usage/configuration.html#confval-language.'
+        ))
         d['language'] = do_prompt(__('Project language'), 'en')
         if d['language'] == 'en':
             d['language'] = None
@@ -300,7 +303,7 @@ def ask_user(d: Dict[str, Any]) -> None:
         print(__('Indicate which of the following Sphinx extensions should be enabled:'))
         d['extensions'] = []
         for name, description in EXTENSIONS.items():
-            if do_prompt('%s: %s (y/n)' % (name, description), 'n', boolean):
+            if do_prompt(f'{name}: {description} (y/n)', 'n', boolean):
                 d['extensions'].append('sphinx.ext.%s' % name)
 
         # Handle conflicting options
@@ -321,10 +324,11 @@ def ask_user(d: Dict[str, Any]) -> None:
     print()
 
 
-def generate(d: Dict, overwrite: bool = True, silent: bool = False, templatedir: str = None
-             ) -> None:
+def generate(
+    d: dict, overwrite: bool = True, silent: bool = False, templatedir: str | None = None
+) -> None:
     """Generate project based on values in *d*."""
-    template = QuickstartRenderer(templatedir=templatedir)
+    template = QuickstartRenderer(templatedir or '')
 
     if 'mastertoctree' not in d:
         d['mastertoctree'] = ''
@@ -357,11 +361,11 @@ def generate(d: Dict, overwrite: bool = True, silent: bool = False, templatedir:
     ensuredir(path.join(srcdir, d['dot'] + 'templates'))
     ensuredir(path.join(srcdir, d['dot'] + 'static'))
 
-    def write_file(fpath: str, content: str, newline: str = None) -> None:
+    def write_file(fpath: str, content: str, newline: str | None = None) -> None:
         if overwrite or not path.isfile(fpath):
             if 'quiet' not in d:
                 print(__('Creating file %s.') % fpath)
-            with open(fpath, 'wt', encoding='utf-8', newline=newline) as f:
+            with open(fpath, 'w', encoding='utf-8', newline=newline) as f:
                 f.write(content)
         else:
             if 'quiet' not in d:
@@ -379,7 +383,7 @@ def generate(d: Dict, overwrite: bool = True, silent: bool = False, templatedir:
     if template._has_custom_template('quickstart/master_doc.rst_t'):
         msg = ('A custom template `master_doc.rst_t` found. It has been renamed to '
                '`root_doc.rst_t`.  Please rename it on your project too.')
-        print(colorize('red', msg))  # RemovedInSphinx60Warning
+        print(colorize('red', msg))
         write_file(masterfile, template.render('quickstart/master_doc.rst_t', d))
     else:
         write_file(masterfile, template.render('quickstart/root_doc.rst_t', d))
@@ -422,7 +426,7 @@ def generate(d: Dict, overwrite: bool = True, silent: bool = False, templatedir:
     print()
 
 
-def valid_dir(d: Dict) -> bool:
+def valid_dir(d: dict) -> bool:
     dir = d['path']
     if not path.exists(dir):
         return True
@@ -536,7 +540,7 @@ def get_parser() -> argparse.ArgumentParser:
     return parser
 
 
-def main(argv: List[str] = sys.argv[1:]) -> int:
+def main(argv: list[str] = sys.argv[1:]) -> int:
     sphinx.locale.setlocale(locale.LC_ALL, '')
     sphinx.locale.init_console(os.path.join(package_dir, 'locale'), 'sphinx')
 
@@ -548,7 +552,7 @@ def main(argv: List[str] = sys.argv[1:]) -> int:
     try:
         args = parser.parse_args(argv)
     except SystemExit as err:
-        return err.code
+        return err.code  # type: ignore[return-value]
 
     d = vars(args)
     # delete None or False value
@@ -602,4 +606,4 @@ def main(argv: List[str] = sys.argv[1:]) -> int:
 
 
 if __name__ == '__main__':
-    sys.exit(main(sys.argv[1:]))
+    raise SystemExit(main())
