@@ -1,6 +1,8 @@
 """The JavaScript domain."""
 
-from typing import Any, Dict, Iterator, List, Optional, Tuple, cast
+from __future__ import annotations
+
+from typing import Any, Iterator, Tuple, cast
 
 from docutils import nodes
 from docutils.nodes import Element, Node
@@ -18,8 +20,8 @@ from sphinx.locale import _, __
 from sphinx.roles import XRefRole
 from sphinx.util import logging
 from sphinx.util.docfields import Field, GroupedField, TypedField
-from sphinx.util.docutils import SphinxDirective
-from sphinx.util.nodes import make_id, make_refnode
+from sphinx.util.docutils import SphinxDirective, switch_source_input
+from sphinx.util.nodes import make_id, make_refnode, nested_parse_with_titles
 from sphinx.util.typing import OptionSpec
 
 logger = logging.getLogger(__name__)
@@ -40,13 +42,14 @@ class JSObject(ObjectDescription[Tuple[str, str]]):
     option_spec: OptionSpec = {
         'noindex': directives.flag,
         'noindexentry': directives.flag,
+        'nocontentsentry': directives.flag,
     }
 
-    def get_display_prefix(self) -> List[Node]:
+    def get_display_prefix(self) -> list[Node]:
         #: what is displayed right before the documentation entry
         return []
 
-    def handle_signature(self, sig: str, signode: desc_signature) -> Tuple[str, str]:
+    def handle_signature(self, sig: str, signode: desc_signature) -> tuple[str, str]:
         """Breaks down construct signatures
 
         Parses out prefix and argument list from construct definition. The
@@ -108,7 +111,18 @@ class JSObject(ObjectDescription[Tuple[str, str]]):
                 _pseudo_parse_arglist(signode, arglist)
         return fullname, prefix
 
-    def add_target_and_index(self, name_obj: Tuple[str, str], sig: str,
+    def _object_hierarchy_parts(self, sig_node: desc_signature) -> tuple[str, ...]:
+        if 'fullname' not in sig_node:
+            return ()
+        modname = sig_node.get('module')
+        fullname = sig_node['fullname']
+
+        if modname:
+            return (modname, *fullname.split('.'))
+        else:
+            return tuple(fullname.split('.'))
+
+    def add_target_and_index(self, name_obj: tuple[str, str], sig: str,
                              signode: desc_signature) -> None:
         mod_name = self.env.ref_context.get('js:module')
         fullname = (mod_name + '.' if mod_name else '') + name_obj[0]
@@ -124,7 +138,7 @@ class JSObject(ObjectDescription[Tuple[str, str]]):
             if indextext:
                 self.indexnode['entries'].append(('single', indextext, node_id, '', None))
 
-    def get_index_text(self, objectname: str, name_obj: Tuple[str, str]) -> str:
+    def get_index_text(self, objectname: str, name_obj: tuple[str, str]) -> str:
         name, obj = name_obj
         if self.objtype == 'function':
             if not obj:
@@ -201,6 +215,25 @@ class JSObject(ObjectDescription[Tuple[str, str]]):
         """
         return fullname.replace('$', '_S_')
 
+    def _toc_entry_name(self, sig_node: desc_signature) -> str:
+        if not sig_node.get('_toc_parts'):
+            return ''
+
+        config = self.env.app.config
+        objtype = sig_node.parent.get('objtype')
+        if config.add_function_parentheses and objtype in {'function', 'method'}:
+            parens = '()'
+        else:
+            parens = ''
+        *parents, name = sig_node['_toc_parts']
+        if config.toc_object_entries_show_parents == 'domain':
+            return sig_node.get('fullname', name) + parens
+        if config.toc_object_entries_show_parents == 'hide':
+            return name + parens
+        if config.toc_object_entries_show_parents == 'all':
+            return '.'.join(parents + [name + parens])
+        return ''
+
 
 class JSCallable(JSObject):
     """Description of a JavaScript function, method or constructor."""
@@ -225,7 +258,7 @@ class JSConstructor(JSCallable):
 
     allow_nesting = True
 
-    def get_display_prefix(self) -> List[Node]:
+    def get_display_prefix(self) -> list[Node]:
         return [addnodes.desc_sig_keyword('class', 'class'),
                 addnodes.desc_sig_space()]
 
@@ -249,19 +282,27 @@ class JSModule(SphinxDirective):
     :param mod_name: Module name
     """
 
-    has_content = False
+    has_content = True
     required_arguments = 1
     optional_arguments = 0
     final_argument_whitespace = False
     option_spec: OptionSpec = {
-        'noindex': directives.flag
+        'noindex': directives.flag,
+        'nocontentsentry': directives.flag,
     }
 
-    def run(self) -> List[Node]:
+    def run(self) -> list[Node]:
         mod_name = self.arguments[0].strip()
         self.env.ref_context['js:module'] = mod_name
         noindex = 'noindex' in self.options
-        ret: List[Node] = []
+
+        content_node: Element = nodes.section()
+        with switch_source_input(self.state, self.content):
+            # necessary so that the child nodes get the right source/line set
+            content_node.document = self.state.document
+            nested_parse_with_titles(self.state, self.content, content_node)
+
+        ret: list[Node] = []
         if not noindex:
             domain = cast(JavaScriptDomain, self.env.get_domain('js'))
 
@@ -278,6 +319,7 @@ class JSModule(SphinxDirective):
             indextext = _('%s (module)') % mod_name
             inode = addnodes.index(entries=[('single', indextext, node_id, '', None)])
             ret.append(inode)
+        ret.extend(content_node.children)
         return ret
 
     def make_old_id(self, modname: str) -> str:
@@ -291,7 +333,7 @@ class JSModule(SphinxDirective):
 
 class JSXRefRole(XRefRole):
     def process_link(self, env: BuildEnvironment, refnode: Element,
-                     has_explicit_title: bool, title: str, target: str) -> Tuple[str, str]:
+                     has_explicit_title: bool, title: str, target: str) -> tuple[str, str]:
         # basically what sphinx.domains.python.PyXRefRole does
         refnode['js:object'] = env.ref_context.get('js:object')
         refnode['js:module'] = env.ref_context.get('js:module')
@@ -338,13 +380,13 @@ class JavaScriptDomain(Domain):
         'attr':  JSXRefRole(),
         'mod':   JSXRefRole(),
     }
-    initial_data: Dict[str, Dict[str, Tuple[str, str]]] = {
+    initial_data: dict[str, dict[str, tuple[str, str]]] = {
         'objects': {},  # fullname -> docname, node_id, objtype
         'modules': {},  # modname  -> docname, node_id
     }
 
     @property
-    def objects(self) -> Dict[str, Tuple[str, str, str]]:
+    def objects(self) -> dict[str, tuple[str, str, str]]:
         return self.data.setdefault('objects', {})  # fullname -> docname, node_id, objtype
 
     def note_object(self, fullname: str, objtype: str, node_id: str,
@@ -356,7 +398,7 @@ class JavaScriptDomain(Domain):
         self.objects[fullname] = (self.env.docname, node_id, objtype)
 
     @property
-    def modules(self) -> Dict[str, Tuple[str, str]]:
+    def modules(self) -> dict[str, tuple[str, str]]:
         return self.data.setdefault('modules', {})  # modname -> docname, node_id
 
     def note_module(self, modname: str, node_id: str) -> None:
@@ -370,7 +412,7 @@ class JavaScriptDomain(Domain):
             if pkg_docname == docname:
                 del self.modules[modname]
 
-    def merge_domaindata(self, docnames: List[str], otherdata: Dict) -> None:
+    def merge_domaindata(self, docnames: list[str], otherdata: dict[str, Any]) -> None:
         # XXX check duplicates
         for fullname, (fn, node_id, objtype) in otherdata['objects'].items():
             if fn in docnames:
@@ -379,8 +421,15 @@ class JavaScriptDomain(Domain):
             if pkg_docname in docnames:
                 self.modules[mod_name] = (pkg_docname, node_id)
 
-    def find_obj(self, env: BuildEnvironment, mod_name: str, prefix: str, name: str,
-                 typ: str, searchorder: int = 0) -> Tuple[str, Tuple[str, str, str]]:
+    def find_obj(
+        self,
+        env: BuildEnvironment,
+        mod_name: str,
+        prefix: str,
+        name: str,
+        typ: str | None,
+        searchorder: int = 0
+    ) -> tuple[str | None, tuple[str, str, str] | None]:
         if name[-2:] == '()':
             name = name[:-2]
 
@@ -405,7 +454,7 @@ class JavaScriptDomain(Domain):
 
     def resolve_xref(self, env: BuildEnvironment, fromdocname: str, builder: Builder,
                      typ: str, target: str, node: pending_xref, contnode: Element
-                     ) -> Optional[Element]:
+                     ) -> Element | None:
         mod_name = node.get('js:module')
         prefix = node.get('js:object')
         searchorder = 1 if node.hasattr('refspecific') else 0
@@ -416,7 +465,7 @@ class JavaScriptDomain(Domain):
 
     def resolve_any_xref(self, env: BuildEnvironment, fromdocname: str, builder: Builder,
                          target: str, node: pending_xref, contnode: Element
-                         ) -> List[Tuple[str, Element]]:
+                         ) -> list[tuple[str, Element]]:
         mod_name = node.get('js:module')
         prefix = node.get('js:object')
         name, obj = self.find_obj(env, mod_name, prefix, target, None, 1)
@@ -425,11 +474,11 @@ class JavaScriptDomain(Domain):
         return [('js:' + self.role_for_objtype(obj[2]),
                  make_refnode(builder, fromdocname, obj[0], obj[1], contnode, name))]
 
-    def get_objects(self) -> Iterator[Tuple[str, str, str, str, str, int]]:
+    def get_objects(self) -> Iterator[tuple[str, str, str, str, str, int]]:
         for refname, (docname, node_id, typ) in list(self.objects.items()):
             yield refname, refname, typ, docname, node_id, 1
 
-    def get_full_qualified_name(self, node: Element) -> str:
+    def get_full_qualified_name(self, node: Element) -> str | None:
         modname = node.get('js:module')
         prefix = node.get('js:object')
         target = node.get('reftarget')
@@ -439,7 +488,7 @@ class JavaScriptDomain(Domain):
             return '.'.join(filter(None, [modname, prefix, target]))
 
 
-def setup(app: Sphinx) -> Dict[str, Any]:
+def setup(app: Sphinx) -> dict[str, Any]:
     app.add_domain(JavaScriptDomain)
 
     return {
