@@ -1,22 +1,21 @@
 """Global creation environment."""
 
+from __future__ import annotations
+
+import functools
 import os
 import pickle
-import warnings
 from collections import defaultdict
 from copy import copy
 from datetime import datetime
 from os import path
-from typing import (TYPE_CHECKING, Any, Callable, Dict, Generator, Iterator, List, Optional,
-                    Set, Tuple, Union)
+from typing import TYPE_CHECKING, Any, Callable, Generator, Iterator
 
-import docutils
 from docutils import nodes
 from docutils.nodes import Node
 
 from sphinx import addnodes
 from sphinx.config import Config
-from sphinx.deprecation import RemovedInSphinx60Warning
 from sphinx.domains import Domain
 from sphinx.environment.adapters.toctree import TocTree
 from sphinx.errors import BuildEnvironmentError, DocumentError, ExtensionError, SphinxError
@@ -34,10 +33,9 @@ if TYPE_CHECKING:
     from sphinx.application import Sphinx
     from sphinx.builders import Builder
 
-
 logger = logging.getLogger(__name__)
 
-default_settings: Dict[str, Any] = {
+default_settings: dict[str, Any] = {
     'auto_id_prefix': 'id',
     'image_loading': 'link',
     'embed_stylesheet': False,
@@ -54,12 +52,10 @@ default_settings: Dict[str, Any] = {
     'file_insertion_enabled': True,
     'smartquotes_locales': [],
 }
-if docutils.__version_info__[:2] <= (0, 17):
-    default_settings['embed_images'] = False
 
 # This is increased every time an environment attribute is added
 # or changed to properly invalidate pickle files.
-ENV_VERSION = 56
+ENV_VERSION = 57
 
 # config status
 CONFIG_OK = 1
@@ -74,11 +70,65 @@ CONFIG_CHANGED_REASON = {
 }
 
 
-versioning_conditions: Dict[str, Union[bool, Callable]] = {
+versioning_conditions: dict[str, bool | Callable] = {
     'none': False,
     'text': is_translatable,
 }
 
+if TYPE_CHECKING:
+    from typing import Literal, MutableMapping
+
+    from typing_extensions import overload
+
+    from sphinx.domains.c import CDomain
+    from sphinx.domains.changeset import ChangeSetDomain
+    from sphinx.domains.citation import CitationDomain
+    from sphinx.domains.cpp import CPPDomain
+    from sphinx.domains.index import IndexDomain
+    from sphinx.domains.javascript import JavaScriptDomain
+    from sphinx.domains.math import MathDomain
+    from sphinx.domains.python import PythonDomain
+    from sphinx.domains.rst import ReSTDomain
+    from sphinx.domains.std import StandardDomain
+    from sphinx.ext.duration import DurationDomain
+    from sphinx.ext.todo import TodoDomain
+
+    class _DomainsType(MutableMapping[str, Domain]):
+        @overload
+        def __getitem__(self, key: Literal["c"]) -> CDomain: ...  # NoQA: E704
+        @overload
+        def __getitem__(self, key: Literal["cpp"]) -> CPPDomain: ...  # NoQA: E704
+        @overload
+        def __getitem__(self, key: Literal["changeset"]) -> ChangeSetDomain: ...  # NoQA: E704
+        @overload
+        def __getitem__(self, key: Literal["citation"]) -> CitationDomain: ...  # NoQA: E704
+        @overload
+        def __getitem__(self, key: Literal["index"]) -> IndexDomain: ...  # NoQA: E704
+        @overload
+        def __getitem__(self, key: Literal["js"]) -> JavaScriptDomain: ...  # NoQA: E704
+        @overload
+        def __getitem__(self, key: Literal["math"]) -> MathDomain: ...  # NoQA: E704
+        @overload
+        def __getitem__(self, key: Literal["py"]) -> PythonDomain: ...  # NoQA: E704
+        @overload
+        def __getitem__(self, key: Literal["rst"]) -> ReSTDomain: ...  # NoQA: E704
+        @overload
+        def __getitem__(self, key: Literal["std"]) -> StandardDomain: ...  # NoQA: E704
+        @overload
+        def __getitem__(self, key: Literal["duration"]) -> DurationDomain: ...  # NoQA: E704
+        @overload
+        def __getitem__(self, key: Literal["todo"]) -> TodoDomain: ...  # NoQA: E704
+        @overload
+        def __getitem__(self, key: str) -> Domain: ...  # NoQA: E704
+        def __getitem__(self, key): raise NotImplementedError  # NoQA: E704
+        def __setitem__(self, key, value): raise NotImplementedError  # NoQA: E704
+        def __delitem__(self, key): raise NotImplementedError  # NoQA: E704
+        def __iter__(self): raise NotImplementedError  # NoQA: E704
+        def __len__(self): raise NotImplementedError  # NoQA: E704
+
+else:
+    _DomainsType = dict
+
 
 class BuildEnvironment:
     """
@@ -87,11 +137,11 @@ class BuildEnvironment:
     transformations to resolve links to them.
     """
 
-    domains: Dict[str, Domain]
+    domains: _DomainsType
 
     # --------- ENVIRONMENT INITIALIZATION -------------------------------------
 
-    def __init__(self, app: "Sphinx" = None):
+    def __init__(self, app: Sphinx):
         self.app: Sphinx = None
         self.doctreedir: str = None
         self.srcdir: str = None
@@ -100,14 +150,14 @@ class BuildEnvironment:
         self.config_status_extra: str = None
         self.events: EventManager = None
         self.project: Project = None
-        self.version: Dict[str, str] = None
+        self.version: dict[str, str] = None
 
         # the method of doctree versioning; see set_versioning_method
-        self.versioning_condition: Union[bool, Callable] = None
+        self.versioning_condition: bool | Callable = None
         self.versioning_compare: bool = None
 
         # all the registered domains, set by the application
-        self.domains = {}
+        self.domains = _DomainsType()
 
         # the docutils settings for building
         self.settings = default_settings.copy()
@@ -118,50 +168,56 @@ class BuildEnvironment:
 
         # docname -> mtime at the time of reading
         # contains all read docnames
-        self.all_docs: Dict[str, float] = {}
+        self.all_docs: dict[str, float] = {}
         # docname -> set of dependent file
         # names, relative to documentation root
-        self.dependencies: Dict[str, Set[str]] = defaultdict(set)
+        self.dependencies: dict[str, set[str]] = defaultdict(set)
         # docname -> set of included file
         # docnames included from other documents
-        self.included: Dict[str, Set[str]] = defaultdict(set)
+        self.included: dict[str, set[str]] = defaultdict(set)
         # docnames to re-read unconditionally on next build
-        self.reread_always: Set[str] = set()
+        self.reread_always: set[str] = set()
+
+        # docname -> pickled doctree
+        self._pickled_doctree_cache: dict[str, bytes] = {}
+
+        # docname -> doctree
+        self._write_doc_doctree_cache: dict[str, nodes.document] = {}
 
         # File metadata
         # docname -> dict of metadata items
-        self.metadata: Dict[str, Dict[str, Any]] = defaultdict(dict)
+        self.metadata: dict[str, dict[str, Any]] = defaultdict(dict)
 
         # TOC inventory
         # docname -> title node
-        self.titles: Dict[str, nodes.title] = {}
+        self.titles: dict[str, nodes.title] = {}
         # docname -> title node; only different if
         # set differently with title directive
-        self.longtitles: Dict[str, nodes.title] = {}
+        self.longtitles: dict[str, nodes.title] = {}
         # docname -> table of contents nodetree
-        self.tocs: Dict[str, nodes.bullet_list] = {}
+        self.tocs: dict[str, nodes.bullet_list] = {}
         # docname -> number of real entries
-        self.toc_num_entries: Dict[str, int] = {}
+        self.toc_num_entries: dict[str, int] = {}
 
         # used to determine when to show the TOC
         # in a sidebar (don't show if it's only one item)
         # docname -> dict of sectionid -> number
-        self.toc_secnumbers: Dict[str, Dict[str, Tuple[int, ...]]] = {}
+        self.toc_secnumbers: dict[str, dict[str, tuple[int, ...]]] = {}
         # docname -> dict of figtype -> dict of figureid -> number
-        self.toc_fignumbers: Dict[str, Dict[str, Dict[str, Tuple[int, ...]]]] = {}
+        self.toc_fignumbers: dict[str, dict[str, dict[str, tuple[int, ...]]]] = {}
 
         # docname -> list of toctree includefiles
-        self.toctree_includes: Dict[str, List[str]] = {}
+        self.toctree_includes: dict[str, list[str]] = {}
         # docname -> set of files (containing its TOCs) to rebuild too
-        self.files_to_rebuild: Dict[str, Set[str]] = {}
+        self.files_to_rebuild: dict[str, set[str]] = {}
         # docnames that have :glob: toctrees
-        self.glob_toctrees: Set[str] = set()
+        self.glob_toctrees: set[str] = set()
         # docnames that have :numbered: toctrees
-        self.numbered_toctrees: Set[str] = set()
+        self.numbered_toctrees: set[str] = set()
 
         # domain-specific inventories, here to be pickled
         # domainname -> domain-specific dict
-        self.domaindata: Dict[str, Dict] = {}
+        self.domaindata: dict[str, dict] = {}
 
         # these map absolute path -> (docnames, unique filename)
         self.images: FilenameUniqDict = FilenameUniqDict()
@@ -169,32 +225,47 @@ class BuildEnvironment:
         self.dlfiles: DownloadFiles = DownloadFiles()
 
         # the original URI for images
-        self.original_image_uri: Dict[str, str] = {}
+        self.original_image_uri: dict[str, str] = {}
 
         # temporary data storage while reading a document
-        self.temp_data: Dict[str, Any] = {}
+        self.temp_data: dict[str, Any] = {}
         # context for cross-references (e.g. current module or class)
         # this is similar to temp_data, but will for example be copied to
         # attributes of "any" cross references
-        self.ref_context: Dict[str, Any] = {}
+        self.ref_context: dict[str, Any] = {}
+
+        # search index data
+
+        # docname -> title
+        self._search_index_titles: dict[str, str] = {}
+        # docname -> filename
+        self._search_index_filenames: dict[str, str] = {}
+        # stemmed words -> set(docname)
+        self._search_index_mapping: dict[str, set[str]] = {}
+        # stemmed words in titles -> set(docname)
+        self._search_index_title_mapping: dict[str, set[str]] = {}
+        # docname -> all titles in document
+        self._search_index_all_titles: dict[str, list[tuple[str, str]]] = {}
+        # docname -> list(index entry)
+        self._search_index_index_entries: dict[str, list[tuple[str, str, str]]] = {}
+        # objtype -> index
+        self._search_index_objtypes: dict[tuple[str, str], int] = {}
+        # objtype index -> (domain, type, objname (localized))
+        self._search_index_objnames: dict[int, tuple[str, str, str]] = {}
 
         # set up environment
-        if app:
-            self.setup(app)
-        else:
-            warnings.warn("The 'app' argument for BuildEnvironment() becomes required now.",
-                          RemovedInSphinx60Warning, stacklevel=2)
+        self.setup(app)
 
-    def __getstate__(self) -> Dict:
+    def __getstate__(self) -> dict:
         """Obtains serializable data for pickling."""
         __dict__ = self.__dict__.copy()
         __dict__.update(app=None, domains={}, events=None)  # clear unpickable attributes
         return __dict__
 
-    def __setstate__(self, state: Dict) -> None:
+    def __setstate__(self, state: dict) -> None:
         self.__dict__.update(state)
 
-    def setup(self, app: "Sphinx") -> None:
+    def setup(self, app: Sphinx) -> None:
         """Set up BuildEnvironment object."""
         if self.version and self.version != app.registry.get_envversion(app):
             raise BuildEnvironmentError(__('build environment version not current'))
@@ -212,7 +283,7 @@ class BuildEnvironment:
         self.version = app.registry.get_envversion(app)
 
         # initialize domains
-        self.domains = {}
+        self.domains = _DomainsType()
         for domain in app.registry.create_domains(self):
             self.domains[domain.name] = domain
 
@@ -240,14 +311,14 @@ class BuildEnvironment:
                 extension = extensions[0]
             else:
                 extension = '%d' % (len(extensions),)
-            self.config_status_extra = ' (%r)' % (extension,)
+            self.config_status_extra = f' ({extension!r})'
         else:
             # check if a config value was changed that affects how
             # doctrees are read
             for item in config.filter('env'):
                 if self.config[item.name] != item.value:
                     self.config_status = CONFIG_CHANGED
-                    self.config_status_extra = ' (%r)' % (item.name,)
+                    self.config_status_extra = f' ({item.name!r})'
                     break
 
         self.config = config
@@ -261,7 +332,7 @@ class BuildEnvironment:
         # Allow to disable by 3rd party extension (workaround)
         self.settings.setdefault('smart_quotes', True)
 
-    def set_versioning_method(self, method: Union[str, Callable], compare: bool) -> None:
+    def set_versioning_method(self, method: str | Callable, compare: bool) -> None:
         """This sets the doctree versioning method for this environment.
 
         Versioning methods are a builder property; only builders with the same
@@ -269,7 +340,7 @@ class BuildEnvironment:
         raise an exception if the user tries to use an environment with an
         incompatible versioning method.
         """
-        condition: Union[bool, Callable]
+        condition: bool | Callable
         if callable(method):
             condition = method
         else:
@@ -294,8 +365,8 @@ class BuildEnvironment:
         for domain in self.domains.values():
             domain.clear_doc(docname)
 
-    def merge_info_from(self, docnames: List[str], other: "BuildEnvironment",
-                        app: "Sphinx") -> None:
+    def merge_info_from(self, docnames: list[str], other: BuildEnvironment,
+                        app: Sphinx) -> None:
         """Merge global information gathered about *docnames* while reading them
         from the *other* environment.
 
@@ -312,7 +383,7 @@ class BuildEnvironment:
             domain.merge_domaindata(docnames, other.domaindata[domainname])
         self.events.emit('env-merge-info', self, docnames, other)
 
-    def path2doc(self, filename: str) -> Optional[str]:
+    def path2doc(self, filename: str) -> str | None:
         """Return the docname for the filename if the file is document.
 
         *filename* should be absolute or relative to the source directory.
@@ -327,7 +398,7 @@ class BuildEnvironment:
         """
         return self.project.doc2path(docname, base)
 
-    def relfn2path(self, filename: str, docname: str = None) -> Tuple[str, str]:
+    def relfn2path(self, filename: str, docname: str | None = None) -> tuple[str, str]:
         """Return paths to a file referenced from a document, relative to
         documentation root and absolute.
 
@@ -340,18 +411,18 @@ class BuildEnvironment:
             rel_fn = filename[1:]
         else:
             docdir = path.dirname(self.doc2path(docname or self.docname,
-                                                base=None))
+                                                base=False))
             rel_fn = path.join(docdir, filename)
 
         return (canon_path(path.normpath(rel_fn)),
                 path.normpath(path.join(self.srcdir, rel_fn)))
 
     @property
-    def found_docs(self) -> Set[str]:
+    def found_docs(self) -> set[str]:
         """contains all existing docnames."""
         return self.project.docnames
 
-    def find_files(self, config: Config, builder: "Builder") -> None:
+    def find_files(self, config: Config, builder: Builder) -> None:
         """Find all source files in the source dir and put them in
         self.found_docs.
         """
@@ -380,13 +451,13 @@ class BuildEnvironment:
             raise DocumentError(__('Failed to scan documents in %s: %r') %
                                 (self.srcdir, exc)) from exc
 
-    def get_outdated_files(self, config_changed: bool) -> Tuple[Set[str], Set[str], Set[str]]:
+    def get_outdated_files(self, config_changed: bool) -> tuple[set[str], set[str], set[str]]:
         """Return (added, changed, removed) sets."""
         # clear all files no longer present
         removed = set(self.all_docs) - self.found_docs
 
-        added: Set[str] = set()
-        changed: Set[str] = set()
+        added: set[str] = set()
+        changed: set[str] = set()
 
         if config_changed:
             # config values affect e.g. substitutions
@@ -437,8 +508,8 @@ class BuildEnvironment:
 
         return added, changed, removed
 
-    def check_dependents(self, app: "Sphinx", already: Set[str]) -> Generator[str, None, None]:
-        to_rewrite: List[str] = []
+    def check_dependents(self, app: Sphinx, already: set[str]) -> Generator[str, None, None]:
+        to_rewrite: list[str] = []
         for docnames in self.events.emit('env-get-updated', self):
             to_rewrite.extend(docnames)
         for docname in set(to_rewrite):
@@ -488,7 +559,9 @@ class BuildEnvironment:
 
         *filename* should be absolute or relative to the source directory.
         """
-        self.included[self.docname].add(self.path2doc(filename))
+        doc = self.path2doc(filename)
+        if doc:
+            self.included[self.docname].add(doc)
 
     def note_reread(self) -> None:
         """Add the current document to the list of documents that will
@@ -510,21 +583,40 @@ class BuildEnvironment:
 
     def get_doctree(self, docname: str) -> nodes.document:
         """Read the doctree for a file from the pickle and return it."""
-        filename = path.join(self.doctreedir, docname + '.doctree')
-        with open(filename, 'rb') as f:
-            doctree = pickle.load(f)
+        try:
+            serialised = self._pickled_doctree_cache[docname]
+        except KeyError:
+            filename = path.join(self.doctreedir, docname + '.doctree')
+            with open(filename, 'rb') as f:
+                serialised = self._pickled_doctree_cache[docname] = f.read()
+
+        doctree = pickle.loads(serialised)
         doctree.settings.env = self
         doctree.reporter = LoggingReporter(self.doc2path(docname))
         return doctree
 
-    def get_and_resolve_doctree(self, docname: str, builder: "Builder",
-                                doctree: nodes.document = None, prune_toctrees: bool = True,
-                                includehidden: bool = False) -> nodes.document:
+    @functools.cached_property
+    def master_doctree(self) -> nodes.document:
+        return self.get_doctree(self.config.root_doc)
+
+    def get_and_resolve_doctree(
+        self,
+        docname: str,
+        builder: Builder,
+        doctree: nodes.document | None = None,
+        prune_toctrees: bool = True,
+        includehidden: bool = False
+    ) -> nodes.document:
         """Read the doctree from the pickle, resolve cross-references and
         toctrees and return it.
         """
         if doctree is None:
-            doctree = self.get_doctree(docname)
+            try:
+                doctree = self._write_doc_doctree_cache.pop(docname)
+                doctree.settings.env = self
+                doctree.reporter = LoggingReporter(self.doc2path(docname))
+            except KeyError:
+                doctree = self.get_doctree(docname)
 
         # resolve all pending cross-references
         self.apply_post_transforms(doctree, docname)
@@ -541,9 +633,9 @@ class BuildEnvironment:
 
         return doctree
 
-    def resolve_toctree(self, docname: str, builder: "Builder", toctree: addnodes.toctree,
+    def resolve_toctree(self, docname: str, builder: Builder, toctree: addnodes.toctree,
                         prune: bool = True, maxdepth: int = 0, titles_only: bool = False,
-                        collapse: bool = False, includehidden: bool = False) -> Node:
+                        collapse: bool = False, includehidden: bool = False) -> Node | None:
         """Resolve a *toctree* node into individual bullet lists with titles
         as items, returning None (if no containing titles are found) or
         a new node.
@@ -560,7 +652,7 @@ class BuildEnvironment:
                                      includehidden)
 
     def resolve_references(self, doctree: nodes.document, fromdocname: str,
-                           builder: "Builder") -> None:
+                           builder: Builder) -> None:
         self.apply_post_transforms(doctree, fromdocname)
 
     def apply_post_transforms(self, doctree: nodes.document, docname: str) -> None:
@@ -580,10 +672,12 @@ class BuildEnvironment:
         # allow custom references to be resolved
         self.events.emit('doctree-resolved', doctree, docname)
 
-    def collect_relations(self) -> Dict[str, List[Optional[str]]]:
+    def collect_relations(self) -> dict[str, list[str | None]]:
         traversed = set()
 
-        def traverse_toctree(parent: str, docname: str) -> Iterator[Tuple[str, str]]:
+        def traverse_toctree(
+            parent: str | None, docname: str
+        ) -> Iterator[tuple[str | None, str]]:
             if parent == docname:
                 logger.warning(__('self referenced toctree found. Ignored.'),
                                location=docname, type='toc',
