@@ -1,9 +1,11 @@
 """Index entries adapters for sphinx.environment."""
 
+from __future__ import annotations
+
 import re
 import unicodedata
 from itertools import groupby
-from typing import Any, Dict, List, Optional, Pattern, Tuple, cast
+from typing import Any, cast
 
 from sphinx.builders import Builder
 from sphinx.domains.index import IndexDomain
@@ -20,13 +22,13 @@ class IndexEntries:
         self.env = env
 
     def create_index(self, builder: Builder, group_entries: bool = True,
-                     _fixre: Pattern = re.compile(r'(.*) ([(][^()]*[)])')
-                     ) -> List[Tuple[str, List[Tuple[str, Any]]]]:
+                     _fixre: re.Pattern = re.compile(r'(.*) ([(][^()]*[)])')
+                     ) -> list[tuple[str, list[tuple[str, Any]]]]:
         """Create the real index from the collected index entries."""
-        new: Dict[str, List] = {}
+        new: dict[str, list] = {}
 
-        def add_entry(word: str, subword: str, main: str, link: bool = True,
-                      dic: Dict[str, List] = new, key: Optional[str] = None) -> None:
+        def add_entry(word: str, subword: str, main: str | None, link: bool = True,
+                      dic: dict[str, list] = new, key: str | None = None) -> None:
             # Force the word to be unicode if it's a ASCII bytestring.
             # This will solve problems with unicode normalization later.
             # For instance the RFC role will add bytestrings at the moment
@@ -79,7 +81,7 @@ class IndexEntries:
                     logger.warning(str(err), location=fn)
 
         # sort the index entries for same keyword.
-        def keyfunc0(entry: Tuple[str, str]) -> Tuple[bool, str]:
+        def keyfunc0(entry: tuple[str, str]) -> tuple[bool, str]:
             main, uri = entry
             return (not main, uri)  # show main entries at first
 
@@ -89,7 +91,7 @@ class IndexEntries:
                 subentry[0].sort(key=keyfunc0)  # type: ignore
 
         # sort the index entries
-        def keyfunc(entry: Tuple[str, List]) -> Tuple[Tuple[int, str], str]:
+        def keyfunc(entry: tuple[str, list]) -> tuple[tuple[int, str], str]:
             key, (void, void, category_key) = entry
             if category_key:
                 # using specified category key to sort
@@ -118,7 +120,7 @@ class IndexEntries:
             #     (in module foo)
             #     (in module bar)
             oldkey = ''
-            oldsubitems: Optional[Dict[str, List]] = None
+            oldsubitems: dict[str, list] | None = None
             i = 0
             while i < len(newlist):
                 key, (targets, subitems, _key) = newlist[i]
@@ -140,7 +142,7 @@ class IndexEntries:
                 i += 1
 
         # sort the sub-index entries
-        def keyfunc2(entry: Tuple[str, List]) -> str:
+        def keyfunc2(entry: tuple[str, list]) -> str:
             key = unicodedata.normalize('NFD', entry[0].lower())
             if key.startswith('\N{RIGHT-TO-LEFT MARK}'):
                 key = key[1:]
@@ -149,7 +151,7 @@ class IndexEntries:
             return key
 
         # group the entries by letter
-        def keyfunc3(item: Tuple[str, List]) -> str:
+        def keyfunc3(item: tuple[str, list]) -> str:
             # hack: mutating the subitems dicts to a list in the keyfunc
             k, v = item
             v[1] = sorted(((si, se) for (si, (se, void, void)) in v[1].items()),
