@@ -9,6 +9,8 @@ Copyright 2008 Société des arts technologiques (SAT),
 https://sat.qc.ca/
 """
 
+from __future__ import annotations
+
 import argparse
 import glob
 import locale
@@ -18,7 +20,7 @@ from copy import copy
 from fnmatch import fnmatch
 from importlib.machinery import EXTENSION_SUFFIXES
 from os import path
-from typing import Any, Generator, List, Optional, Tuple
+from typing import Any, Generator
 
 import sphinx.locale
 from sphinx import __display_version__, package_dir
@@ -46,11 +48,10 @@ template_dir = path.join(package_dir, 'templates', 'apidoc')
 def is_initpy(filename: str) -> bool:
     """Check *filename* is __init__ file or not."""
     basename = path.basename(filename)
-    for suffix in sorted(PY_SUFFIXES, key=len, reverse=True):
-        if basename == '__init__' + suffix:
-            return True
-    else:
-        return False
+    return any(
+        basename == '__init__' + suffix
+        for suffix in sorted(PY_SUFFIXES, key=len, reverse=True)
+    )
 
 
 def module_join(*modnames: str) -> str:
@@ -58,7 +59,7 @@ def module_join(*modnames: str) -> str:
     return '.'.join(filter(None, modnames))
 
 
-def is_packagedir(dirname: Optional[str] = None, files: Optional[List[str]] = None) -> bool:
+def is_packagedir(dirname: str | None = None, files: list[str] | None = None) -> bool:
     """Check given *files* contains __init__ file."""
     if files is None and dirname is None:
         return False
@@ -72,7 +73,7 @@ def write_file(name: str, text: str, opts: Any) -> None:
     """Write the output file for module/package <name>."""
     quiet = getattr(opts, 'quiet', None)
 
-    fname = path.join(opts.destdir, '%s.%s' % (name, opts.suffix))
+    fname = path.join(opts.destdir, f'{name}.{opts.suffix}')
     if opts.dryrun:
         if not quiet:
             print(__('Would create file %s.') % fname)
@@ -88,7 +89,7 @@ def write_file(name: str, text: str, opts: Any) -> None:
 
 
 def create_module_file(package: str, basename: str, opts: Any,
-                       user_template_dir: Optional[str] = None) -> None:
+                       user_template_dir: str | None = None) -> None:
     """Build the text of the file and write the file."""
     options = copy(OPTIONS)
     if opts.includeprivate and 'private-members' not in options:
@@ -105,9 +106,9 @@ def create_module_file(package: str, basename: str, opts: Any,
     write_file(qualname, text, opts)
 
 
-def create_package_file(root: str, master_package: str, subroot: str, py_files: List[str],
-                        opts: Any, subs: List[str], is_namespace: bool,
-                        excludes: List[str] = [], user_template_dir: Optional[str] = None
+def create_package_file(root: str, master_package: str, subroot: str, py_files: list[str],
+                        opts: Any, subs: list[str], is_namespace: bool,
+                        excludes: list[str] = [], user_template_dir: str | None = None
                         ) -> None:
     """Build the text of the file and write the file."""
     # build a list of sub packages (directories containing an __init__ file)
@@ -145,8 +146,8 @@ def create_package_file(root: str, master_package: str, subroot: str, py_files:
             create_module_file(None, submodule, opts, user_template_dir)
 
 
-def create_modules_toc_file(modules: List[str], opts: Any, name: str = 'modules',
-                            user_template_dir: Optional[str] = None) -> None:
+def create_modules_toc_file(modules: list[str], opts: Any, name: str = 'modules',
+                            user_template_dir: str | None = None) -> None:
     """Create the module's index."""
     modules.sort()
     prev_module = ''
@@ -166,7 +167,7 @@ def create_modules_toc_file(modules: List[str], opts: Any, name: str = 'modules'
     write_file(name, text, opts)
 
 
-def is_skipped_package(dirname: str, opts: Any, excludes: List[str] = []) -> bool:
+def is_skipped_package(dirname: str, opts: Any, excludes: list[str] = []) -> bool:
     """Check if we want to skip this module."""
     if not path.isdir(dirname):
         return False
@@ -178,14 +179,10 @@ def is_skipped_package(dirname: str, opts: Any, excludes: List[str] = []) -> boo
         return True
 
     # Check there is some showable module inside package
-    if all(is_excluded(path.join(dirname, f), excludes) for f in files):
-        # all submodules are excluded
-        return True
-    else:
-        return False
+    return all(is_excluded(path.join(dirname, f), excludes) for f in files)
 
 
-def is_skipped_module(filename: str, opts: Any, excludes: List[str]) -> bool:
+def is_skipped_module(filename: str, opts: Any, excludes: list[str]) -> bool:
     """Check if we want to skip this module."""
     if not path.exists(filename):
         # skip if the file doesn't exist
@@ -197,8 +194,8 @@ def is_skipped_module(filename: str, opts: Any, excludes: List[str]) -> bool:
         return False
 
 
-def walk(rootpath: str, excludes: List[str], opts: Any
-         ) -> Generator[Tuple[str, List[str], List[str]], None, None]:
+def walk(rootpath: str, excludes: list[str], opts: Any
+         ) -> Generator[tuple[str, list[str], list[str]], None, None]:
     """Walk through the directory and list files and subdirectories up."""
     followlinks = getattr(opts, 'followlinks', False)
     includeprivate = getattr(opts, 'includeprivate', False)
@@ -212,7 +209,7 @@ def walk(rootpath: str, excludes: List[str], opts: Any
         # remove hidden ('.') and private ('_') directories, as well as
         # excluded dirs
         if includeprivate:
-            exclude_prefixes: Tuple[str, ...] = ('.',)
+            exclude_prefixes: tuple[str, ...] = ('.',)
         else:
             exclude_prefixes = ('.', '_')
 
@@ -222,17 +219,16 @@ def walk(rootpath: str, excludes: List[str], opts: Any
         yield root, subs, files
 
 
-def has_child_module(rootpath: str, excludes: List[str], opts: Any) -> bool:
+def has_child_module(rootpath: str, excludes: list[str], opts: Any) -> bool:
     """Check the given directory contains child module/s (at least one)."""
-    for _root, _subs, files in walk(rootpath, excludes, opts):
-        if files:
-            return True
-
-    return False
+    return any(
+        files
+        for _root, _subs, files in walk(rootpath, excludes, opts)
+    )
 
 
-def recurse_tree(rootpath: str, excludes: List[str], opts: Any,
-                 user_template_dir: Optional[str] = None) -> List[str]:
+def recurse_tree(rootpath: str, excludes: list[str], opts: Any,
+                 user_template_dir: str | None = None) -> list[str]:
     """
     Look for every file in the directory tree and create the corresponding
     ReST files.
@@ -285,16 +281,16 @@ def recurse_tree(rootpath: str, excludes: List[str], opts: Any,
     return toplevels
 
 
-def is_excluded(root: str, excludes: List[str]) -> bool:
+def is_excluded(root: str, excludes: list[str]) -> bool:
     """Check if the directory is in the exclude list.
 
     Note: by having trailing slashes, we avoid common prefix issues, like
           e.g. an exclude "foo" also accidentally excluding "foobar".
     """
-    for exclude in excludes:
-        if fnmatch(root, exclude):
-            return True
-    return False
+    return any(
+        fnmatch(root, exclude)
+        for exclude in excludes
+    )
 
 
 def get_parser() -> argparse.ArgumentParser:
@@ -394,7 +390,7 @@ Note: By default this script will not overwrite already created files."""))
     return parser
 
 
-def main(argv: List[str] = sys.argv[1:]) -> int:
+def main(argv: list[str] = sys.argv[1:]) -> int:
     """Parse and check the command line arguments."""
     sphinx.locale.setlocale(locale.LC_ALL, '')
     sphinx.locale.init_console(os.path.join(package_dir, 'locale'), 'sphinx')
@@ -412,7 +408,7 @@ def main(argv: List[str] = sys.argv[1:]) -> int:
         args.suffix = args.suffix[1:]
     if not path.isdir(rootpath):
         print(__('%s is not a directory.') % rootpath, file=sys.stderr)
-        sys.exit(1)
+        raise SystemExit(1)
     if not args.dryrun:
         ensuredir(args.destdir)
     excludes = [path.abspath(exclude) for exclude in args.exclude_pattern]
