@@ -1,45 +1,40 @@
-r"""
-    sphinx.ext.inheritance_diagram
-    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+r"""Defines a docutils directive for inserting inheritance diagrams.
 
-    Defines a docutils directive for inserting inheritance diagrams.
+Provide the directive with one or more classes or modules (separated
+by whitespace).  For modules, all of the classes in that module will
+be used.
 
-    Provide the directive with one or more classes or modules (separated
-    by whitespace).  For modules, all of the classes in that module will
-    be used.
+Example::
 
-    Example::
+   Given the following classes:
 
-       Given the following classes:
+   class A: pass
+   class B(A): pass
+   class C(A): pass
+   class D(B, C): pass
+   class E(B): pass
 
-       class A: pass
-       class B(A): pass
-       class C(A): pass
-       class D(B, C): pass
-       class E(B): pass
+   .. inheritance-diagram: D E
 
-       .. inheritance-diagram: D E
+   Produces a graph like the following:
 
-       Produces a graph like the following:
+               A
+              / \
+             B   C
+            / \ /
+           E   D
 
-                   A
-                  / \
-                 B   C
-                / \ /
-               E   D
-
-    The graph is inserted as a PNG+image map into HTML and a PDF in
-    LaTeX.
-
-    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
+The graph is inserted as a PNG+image map into HTML and a PDF in
+LaTeX.
 """
 
+from __future__ import annotations
+
 import builtins
 import inspect
 import re
 from importlib import import_module
-from typing import Any, Dict, Iterable, List, Tuple, cast
+from typing import Any, Iterable, cast
 
 from docutils import nodes
 from docutils.nodes import Node
@@ -53,7 +48,8 @@ from sphinx.ext.graphviz import (figure_wrapper, graphviz, render_dot_html, rend
                                  render_dot_texinfo)
 from sphinx.util import md5
 from sphinx.util.docutils import SphinxDirective
-from sphinx.writers.html import HTMLTranslator
+from sphinx.util.typing import OptionSpec
+from sphinx.writers.html import HTML5Translator
 from sphinx.writers.latex import LaTeXTranslator
 from sphinx.writers.texinfo import TexinfoTranslator
 
@@ -136,9 +132,10 @@ class InheritanceGraph:
     from all the way to the root "object", and then is able to generate a
     graphviz dot graph from them.
     """
-    def __init__(self, class_names: List[str], currmodule: str, show_builtins: bool = False,
-                 private_bases: bool = False, parts: int = 0, aliases: Dict[str, str] = None,
-                 top_classes: List[Any] = []) -> None:
+    def __init__(self, class_names: list[str], currmodule: str, show_builtins: bool = False,
+                 private_bases: bool = False, parts: int = 0,
+                 aliases: dict[str, str] | None = None, top_classes: list[Any] = []
+                 ) -> None:
         """*class_names* is a list of child classes to show bases from.
 
         If *show_builtins* is True, then Python builtins will be shown
@@ -152,16 +149,16 @@ class InheritanceGraph:
             raise InheritanceException('No classes found for '
                                        'inheritance diagram')
 
-    def _import_classes(self, class_names: List[str], currmodule: str) -> List[Any]:
+    def _import_classes(self, class_names: list[str], currmodule: str) -> list[Any]:
         """Import a list of classes."""
-        classes = []  # type: List[Any]
+        classes: list[Any] = []
         for name in class_names:
             classes.extend(import_classes(name, currmodule))
         return classes
 
-    def _class_info(self, classes: List[Any], show_builtins: bool, private_bases: bool,
-                    parts: int, aliases: Dict[str, str], top_classes: List[Any]
-                    ) -> List[Tuple[str, str, List[str], str]]:
+    def _class_info(self, classes: list[Any], show_builtins: bool, private_bases: bool,
+                    parts: int, aliases: dict[str, str], top_classes: list[Any]
+                    ) -> list[tuple[str, str, list[str], str]]:
         """Return name and bases for all classes that are ancestors of
         *classes*.
 
@@ -198,7 +195,7 @@ class InheritanceGraph:
             except Exception:  # might raise AttributeError for strange classes
                 pass
 
-            baselist = []  # type: List[str]
+            baselist: list[str] = []
             all_classes[cls] = (nodename, fullname, baselist, tooltip)
 
             if fullname in top_classes:
@@ -218,7 +215,9 @@ class InheritanceGraph:
 
         return list(all_classes.values())
 
-    def class_name(self, cls: Any, parts: int = 0, aliases: Dict[str, str] = None) -> str:
+    def class_name(
+        self, cls: Any, parts: int = 0, aliases: dict[str, str] | None = None
+    ) -> str:
         """Given a class object, return a fully-qualified name.
 
         This works for things I've tested in matplotlib so far, but may not be
@@ -228,7 +227,7 @@ class InheritanceGraph:
         if module in ('__builtin__', 'builtins'):
             fullname = cls.__name__
         else:
-            fullname = '%s.%s' % (module, cls.__qualname__)
+            fullname = f'{module}.{cls.__qualname__}'
         if parts == 0:
             result = fullname
         else:
@@ -238,7 +237,7 @@ class InheritanceGraph:
             return aliases[result]
         return result
 
-    def get_all_class_names(self) -> List[str]:
+    def get_all_class_names(self) -> list[str]:
         """Get all of the class names involved in the graph."""
         return [fullname for (_, fullname, _, _) in self.class_info]
 
@@ -262,14 +261,15 @@ class InheritanceGraph:
         'style': '"setlinewidth(0.5)"',
     }
 
-    def _format_node_attrs(self, attrs: Dict) -> str:
+    def _format_node_attrs(self, attrs: dict[str, Any]) -> str:
         return ','.join(['%s=%s' % x for x in sorted(attrs.items())])
 
-    def _format_graph_attrs(self, attrs: Dict) -> str:
+    def _format_graph_attrs(self, attrs: dict[str, Any]) -> str:
         return ''.join(['%s=%s;\n' % x for x in sorted(attrs.items())])
 
-    def generate_dot(self, name: str, urls: Dict = {}, env: BuildEnvironment = None,
-                     graph_attrs: Dict = {}, node_attrs: Dict = {}, edge_attrs: Dict = {}
+    def generate_dot(self, name: str, urls: dict[str, str] = {},
+                     env: BuildEnvironment | None = None,
+                     graph_attrs: dict = {}, node_attrs: dict = {}, edge_attrs: dict = {}
                      ) -> str:
         """Generate a graphviz dot graph from the classes that were passed in
         to __init__.
@@ -292,7 +292,7 @@ class InheritanceGraph:
             n_attrs.update(env.config.inheritance_node_attrs)
             e_attrs.update(env.config.inheritance_edge_attrs)
 
-        res = []  # type: List[str]
+        res: list[str] = []
         res.append('digraph %s {\n' % name)
         res.append(self._format_graph_attrs(g_attrs))
 
@@ -331,14 +331,14 @@ class InheritanceDiagram(SphinxDirective):
     required_arguments = 1
     optional_arguments = 0
     final_argument_whitespace = True
-    option_spec = {
+    option_spec: OptionSpec = {
         'parts': int,
         'private-bases': directives.flag,
         'caption': directives.unchanged,
         'top-classes': directives.unchanged_required,
     }
 
-    def run(self) -> List[Node]:
+    def run(self) -> list[Node]:
         node = inheritance_diagram()
         node.document = self.state.document
         class_names = self.arguments[0].split()
@@ -389,7 +389,7 @@ def get_graph_hash(node: inheritance_diagram) -> str:
     return md5(encoded).hexdigest()[-10:]
 
 
-def html_visit_inheritance_diagram(self: HTMLTranslator, node: inheritance_diagram) -> None:
+def html_visit_inheritance_diagram(self: HTML5Translator, node: inheritance_diagram) -> None:
     """
     Output the graph for HTML.  This will insert a PNG with clickable
     image map.
@@ -457,7 +457,7 @@ def skip(self: nodes.NodeVisitor, node: inheritance_diagram) -> None:
     raise nodes.SkipNode
 
 
-def setup(app: Sphinx) -> Dict[str, Any]:
+def setup(app: Sphinx) -> dict[str, Any]:
     app.setup_extension('sphinx.ext.graphviz')
     app.add_node(
         inheritance_diagram,
