@@ -1,26 +1,15 @@
-# -*- coding: utf-8 -*-
-"""
-    sphinx.search.zh
-    ~~~~~~~~~~~~~~~~
+"""Chinese search language: includes routine to split words."""
 
-    Chinese search language: includes routine to split words.
-
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+from __future__ import annotations
 
 import os
 import re
+from typing import Dict, List
+
+import snowballstemmer
 
 from sphinx.search import SearchLanguage
 
-try:
-    from Stemmer import Stemmer as PyStemmer
-    PYSTEMMER = True
-except ImportError:
-    from sphinx.util.stemmer import PorterStemmer
-    PYSTEMMER = False
-
 try:
     import jieba
     JIEBA = True
@@ -235,41 +224,39 @@ class SearchChinese(SearchLanguage):
     language_name = 'Chinese'
     js_stemmer_code = js_porter_stemmer
     stopwords = english_stopwords
-    latin1_letters = re.compile(r'\w+(?u)[\u0000-\u00ff]')
+    latin1_letters = re.compile(r'[a-zA-Z0-9_]+')
+    latin_terms: list[str] = []
 
-    def init(self, options):
+    def init(self, options: dict) -> None:
         if JIEBA:
             dict_path = options.get('dict')
             if dict_path and os.path.isfile(dict_path):
-                jieba.set_dictionary(dict_path)
+                jieba.load_userdict(dict_path)
 
-        if PYSTEMMER:
-            class Stemmer(object):
-                def __init__(self):
-                    self.stemmer = PyStemmer('porter')
+        self.stemmer = snowballstemmer.stemmer('english')
 
-                def stem(self, word):
-                    return self.stemmer.stemWord(word)
-        else:
-            class Stemmer(PorterStemmer):
-                """All those porter stemmer implementations look hideous;
-                make at least the stem method nicer.
-                """
-                def stem(self, word):
-                    return PorterStemmer.stem(self, word, 0, len(word) - 1)
-
-        self.stemmer = Stemmer()
-
-    def split(self, input):
-        chinese = []
+    def split(self, input: str) -> list[str]:
+        chinese: list[str] = []
         if JIEBA:
             chinese = list(jieba.cut_for_search(input))
 
-        latin1 = self.latin1_letters.findall(input)
+        latin1 = \
+            [term.strip() for term in self.latin1_letters.findall(input)]
+        self.latin_terms.extend(latin1)
         return chinese + latin1
 
-    def word_filter(self, stemmed_word):
+    def word_filter(self, stemmed_word: str) -> bool:
         return len(stemmed_word) > 1
 
-    def stem(self, word):
-        return self.stemmer.stem(word)
+    def stem(self, word: str) -> str:
+        # Don't stem Latin words that are long enough to be relevant for search
+        # if not stemmed, but would be too short after being stemmed
+        # avoids some issues with acronyms
+        should_not_be_stemmed = (
+            word in self.latin_terms and
+            len(word) >= 3 and
+            len(self.stemmer.stemWord(word.lower())) < 3
+        )
+        if should_not_be_stemmed:
+            return word.lower()
+        return self.stemmer.stemWord(word.lower())
