@@ -1,11 +1,12 @@
 """Sphinx test fixtures for pytest"""
 
+from __future__ import annotations
+
 import subprocess
 import sys
 from collections import namedtuple
 from io import StringIO
-from subprocess import PIPE
-from typing import Any, Callable, Dict, Generator, Tuple
+from typing import Any, Callable, Generator
 
 import pytest
 
@@ -28,12 +29,12 @@ def pytest_configure(config):
 
 
 @pytest.fixture(scope='session')
-def rootdir() -> str:
+def rootdir() -> str | None:
     return None
 
 
 class SharedResult:
-    cache: Dict[str, Dict[str, str]] = {}
+    cache: dict[str, dict[str, str]] = {}
 
     def store(self, key: str, app_: SphinxTestApp) -> Any:
         if key in self.cache:
@@ -44,7 +45,7 @@ class SharedResult:
         }
         self.cache[key] = data
 
-    def restore(self, key: str) -> Dict[str, StringIO]:
+    def restore(self, key: str) -> dict[str, StringIO]:
         if key not in self.cache:
             return {}
         data = self.cache[key]
@@ -55,8 +56,8 @@ class SharedResult:
 
 
 @pytest.fixture
-def app_params(request: Any, test_params: Dict, shared_result: SharedResult,
-               sphinx_test_tempdir: str, rootdir: str) -> Tuple[Dict, Dict]:
+def app_params(request: Any, test_params: dict, shared_result: SharedResult,
+               sphinx_test_tempdir: str, rootdir: str) -> tuple[dict, dict]:
     """
     Parameters that are specified by 'pytest.mark.sphinx' for
     sphinx.application.Sphinx initialization
@@ -65,7 +66,7 @@ def app_params(request: Any, test_params: Dict, shared_result: SharedResult,
     # ##### process pytest.mark.sphinx
 
     pargs = {}
-    kwargs: Dict[str, Any] = {}
+    kwargs: dict[str, Any] = {}
 
     # to avoid stacking positional args
     for info in reversed(list(request.node.iter_markers("sphinx"))):
@@ -98,7 +99,7 @@ def app_params(request: Any, test_params: Dict, shared_result: SharedResult,
 
 
 @pytest.fixture
-def test_params(request: Any) -> Dict:
+def test_params(request: Any) -> dict:
     """
     Test parameters that are specified by 'pytest.mark.test_params'
 
@@ -122,7 +123,7 @@ def test_params(request: Any) -> Dict:
 
 
 @pytest.fixture(scope='function')
-def app(test_params: Dict, app_params: Tuple[Dict, Dict], make_app: Callable,
+def app(test_params: dict, app_params: tuple[dict, dict], make_app: Callable,
         shared_result: SharedResult) -> Generator[SphinxTestApp, None, None]:
     """
     Provides the 'sphinx.application.Sphinx' object
@@ -159,7 +160,7 @@ def warning(app: SphinxTestApp) -> StringIO:
 
 
 @pytest.fixture()
-def make_app(test_params: Dict, monkeypatch: Any) -> Generator[Callable, None, None]:
+def make_app(test_params: dict, monkeypatch: Any) -> Generator[Callable, None, None]:
     """
     Provides make_app function to initialize SphinxTestApp instance.
     if you want to initialize 'app' in your test function. please use this
@@ -205,7 +206,7 @@ def if_graphviz_found(app: SphinxTestApp) -> None:
     graphviz_dot = getattr(app.config, 'graphviz_dot', '')
     try:
         if graphviz_dot:
-            subprocess.run([graphviz_dot, '-V'], stdout=PIPE, stderr=PIPE)  # show version
+            subprocess.run([graphviz_dot, '-V'], capture_output=True)  # show version
             return
     except OSError:  # No such file or directory
         pass
@@ -214,7 +215,7 @@ def if_graphviz_found(app: SphinxTestApp) -> None:
 
 
 @pytest.fixture(scope='session')
-def sphinx_test_tempdir(tmpdir_factory: Any) -> "util.path":
+def sphinx_test_tempdir(tmpdir_factory: Any) -> util.path:
     """
     Temporary directory wrapped with `path` class.
     """
@@ -223,7 +224,7 @@ def sphinx_test_tempdir(tmpdir_factory: Any) -> "util.path":
 
 
 @pytest.fixture
-def tempdir(tmpdir: str) -> "util.path":
+def tempdir(tmpdir: str) -> util.path:
     """
     Temporary directory wrapped with `path` class.
     This fixture is for back-compatibility with old test implementation.
