@@ -1,24 +1,19 @@
-"""
-    sphinx.transforms.post_transforms.images
-    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+"""Docutils transforms used by Sphinx."""
 
-    Docutils transforms used by Sphinx.
-
-    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+from __future__ import annotations
 
 import os
 import re
 from math import ceil
-from typing import Any, Dict, List, Optional, Tuple
+from typing import Any
 
 from docutils import nodes
 
 from sphinx.application import Sphinx
 from sphinx.locale import __
 from sphinx.transforms import SphinxTransform
-from sphinx.util import epoch_to_rfc1123, logging, requests, rfc1123_to_epoch, sha1
+from sphinx.util import logging, requests, sha1
+from sphinx.util.http_date import epoch_to_rfc1123, rfc1123_to_epoch
 from sphinx.util.images import get_image_extension, guess_mimetype, parse_data_uri
 from sphinx.util.osutil import ensuredir
 
@@ -30,7 +25,7 @@ CRITICAL_PATH_CHAR_RE = re.compile('[:;<>|*" ]')
 
 class BaseImageConverter(SphinxTransform):
     def apply(self, **kwargs: Any) -> None:
-        for node in self.document.traverse(nodes.image):
+        for node in self.document.findall(nodes.image):
             if self.match(node):
                 self.handle(node)
 
@@ -154,13 +149,13 @@ class ImageConverter(BaseImageConverter):
     """A base class for image converters.
 
     An image converter is kind of Docutils transform module.  It is used to
-    convert image files which does not supported by builder to appropriate
-    format for that builder.
+    convert image files which are not supported by a builder to the
+    appropriate format for that builder.
 
     For example, :py:class:`LaTeX builder <.LaTeXBuilder>` supports PDF,
     PNG and JPEG as image formats.  However it does not support SVG images.
-    For such case, to use image converters allows to embed these
-    unsupported images into the document.  One of image converters;
+    For such case, using image converters allows to embed these
+    unsupported images into the document.  One of the image converters;
     :ref:`sphinx.ext.imgconverter <sphinx.ext.imgconverter>` can convert
     a SVG image to PNG format using Imagemagick internally.
 
@@ -178,7 +173,7 @@ class ImageConverter(BaseImageConverter):
     #:
     #: .. todo:: This should be refactored not to store the state without class
     #:           variable.
-    available: Optional[bool] = None
+    available: bool | None = None
 
     #: A conversion rules the image converter supports.
     #: It is represented as a list of pair of source image format (mimetype) and
@@ -189,7 +184,7 @@ class ImageConverter(BaseImageConverter):
     #:         ('image/gif', 'image/png'),
     #:         ('application/pdf', 'image/png'),
     #:     ]
-    conversion_rules: List[Tuple[str, str]] = []
+    conversion_rules: list[tuple[str, str]] = []
 
     def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
@@ -197,6 +192,8 @@ class ImageConverter(BaseImageConverter):
     def match(self, node: nodes.image) -> bool:
         if not self.app.builder.supported_image_types:
             return False
+        elif '?' in node['candidates']:
+            return False
         elif set(self.guess_mimetypes(node)) & set(self.app.builder.supported_image_types):
             # builder supports the image; no need to convert
             return False
@@ -208,12 +205,9 @@ class ImageConverter(BaseImageConverter):
             return False
         else:
             rule = self.get_conversion_rule(node)
-            if rule:
-                return True
-            else:
-                return False
+            return bool(rule)
 
-    def get_conversion_rule(self, node: nodes.image) -> Tuple[str, str]:
+    def get_conversion_rule(self, node: nodes.image) -> tuple[str, str]:
         for candidate in self.guess_mimetypes(node):
             for supported in self.app.builder.supported_image_types:
                 rule = (candidate, supported)
@@ -226,7 +220,7 @@ class ImageConverter(BaseImageConverter):
         """Return the image converter is available or not."""
         raise NotImplementedError()
 
-    def guess_mimetypes(self, node: nodes.image) -> List[str]:
+    def guess_mimetypes(self, node: nodes.image) -> list[str]:
         if '?' in node['candidates']:
             return []
         elif '*' in node['candidates']:
@@ -258,15 +252,15 @@ class ImageConverter(BaseImageConverter):
             self.env.images.add_file(self.env.docname, destpath)
 
     def convert(self, _from: str, _to: str) -> bool:
-        """Convert a image file to expected format.
+        """Convert an image file to the expected format.
 
-        *_from* is a path for source image file, and *_to* is a path for
-        destination file.
+        *_from* is a path of the source image file, and *_to* is a path
+        of the destination file.
         """
         raise NotImplementedError()
 
 
-def setup(app: Sphinx) -> Dict[str, Any]:
+def setup(app: Sphinx) -> dict[str, Any]:
     app.add_post_transform(ImageDownloader)
     app.add_post_transform(DataURIExtractor)
 
